import subprocess
from .settings import get_option
from .log import exception

def execute(command: str, capture: bool = False, allow_single_quotes: bool = False) -> str | None:
    """
    Execute the given code.

    :param code: The code to execute.
    :param capture: Whether to capture the output or simply print it.
    :return: The output of the code if capture is True.
    """
    if not capture:
        try:
            subprocess.run(command, shell=True).check_returncode()
        except subprocess.CalledProcessError as e:
            exception(f"Error executing command `{command}`", e)
            raise e
        return

    with subprocess.Popen(command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE) as proc:
        proc.wait()
        stdout, stderr = proc.communicate()
        error_output = stderr.decode()
        return_code = proc.returncode
        if return_code != 0:
            exception("Error executing command", RuntimeError(error_output))
            raise RuntimeError(f"Error executing command `{command}`: {error_output}")
        return stdout.decode()

def nix_exec(command: str, capture: bool = False, packages: list[str] | None = None) -> str:
    if '"' in command:
        raise ValueError("Double quotes are not allowed in nix_exec")
    packages = packages or []
    packages += ["bash"]
    packages_str = " ".join(packages)
    return execute(f"nix-shell --packages {packages_str} --quiet --run \"{command}\"", capture=capture) or ''



from rich import print
LIGHT_BLUE = "#7EBAE4"
LIGHT_RED = "#e47e7e"
LIGHT_YELLOW = "#e4d97e"
LIGHT_GREEN = "#7ee48e"
LIGHT_PURPLE = "#bd7ee4"
DARK_BLUE = "#5277C3"
NIXOS_LOGO = ""
COLORED_LOGO = f"[{DARK_BLUE}]{NIXOS_LOGO}[/]"

def banner():
    print(f"[{LIGHT_GREEN}] nixos-wizard[/] [{LIGHT_PURPLE}]v0.1.0[/]")

def info(msg: str, *args, **kwargs):
    print(f"[{LIGHT_BLUE}] [/]" + msg, *args, **kwargs)

def error(msg: str, *args, **kwargs):
    print(f"[{LIGHT_RED}]󱗗 [/]" + msg, *args, **kwargs)

def warning(msg: str, *args, **kwargs):
    print(f"[{LIGHT_YELLOW}] [/]" + msg, *args, **kwargs)

def success(msg: str, *args, **kwargs):
    print(f"[{LIGHT_GREEN}]󰗠 [/]{msg}", *args, **kwargs)

def yesno(msg: str, *args, **kwargs) -> bool:
    print(f"[{LIGHT_PURPLE}]󰜴 [/]{msg} \\[{LIGHT_GREEN}]Yy[/]/[{LIGHT_RED}]Nn[/]]", end=" ", *args, **kwargs)
    return input().lower().strip() in ["Y", "y"]

def exception(msg: str, exception: Exception, *args, **kwargs):
    print(f"[{LIGHT_RED}] [/]{msg}: {exception}", *args, **kwargs)



from pathlib import Path
import yaml

HOME_DIR = Path.home()
CONFIG_DIR = HOME_DIR / ".config/" / "nixos-wizard"
CONFIG_PATH = CONFIG_DIR / "config.yaml"

# default settings
DEFAULTS = {
    "dotfiles": HOME_DIR / "dotfiles",
}

def _load_config(config_path: Path) -> dict:
    """
    Load the configuration file.
    """
    with open(config_path, "r") as f:
        data = yaml.safe_load(f)
        for key, value in DEFAULTS.items():
            if key not in data:
                data[key] = value
    return data

data = _load_config(CONFIG_PATH)
def get_option(key: str) -> str:
    """
    Get a value from the configuration.
    """
    return data[key]



from setuptools import setup, find_packages
from pathlib import Path

REPO_DIR = Path.cwd()

with open(REPO_DIR / "README.md", "r") as f:
    long_description = f.read()

setup(
    name="nixos-wizard",
    version="0.1.0",
    license="MIT",
    packages=find_packages(),
    description="simple nixos-wizard",
    install_requires=[
        "rich",
        "pyyaml"
    ],
    entry_points={
        "console_scripts": [
            "nixos-wizard-rebuild=wizard.__main__:rebuild",
            "nixos-wizard-cleanup=wizard.__main__:cleanup",
            "nixos-wizard-update=wizard.__main__:update",
        ]
    }
)



from argparse import ArgumentParser
from .log import info, success, error, warning, yesno, banner
from .exec import nix_exec
from .settings import get_option

def _prepare_rebuild():
    dotfiles = get_option("dotfiles")
    # check that no changes have been made
    diff: str = nix_exec(f"cd {dotfiles}; git status --porcelain", capture=True, packages=["git"])
    if diff.strip():
        # if changes have been made, ask wether to add and commit them
        warning("Changes have been made to the configuration. Commit them before rebuilding.")
        if yesno("Add and commit changes?"):
            nix_exec(f"cd {dotfiles}; git add .", packages=["git"])
            commit_message = nix_exec(f"cd {dotfiles}; nixos-rebuild list-generations | grep current", capture=True)
            info(f"Committing changes with message: {commit_message}")
            nix_exec(f"cd {dotfiles}; git commit -m '{commit_message}'", packages=["git"])
            success("Changes committed.")
        else:
            return

def rebuild():
    banner()
    dotfiles = get_option("dotfiles")
    info("Rebuilding NixOS configuration...")
    _prepare_rebuild()
    # rebuild the configuration
    rebuild_cmd = f"cd {dotfiles}; sudo /usr/bin/env bash -p -c 'nixos-rebuild switch --flake .# --log-format internal-json -v |& nom --json'"
    rebuild_packages = ["nix-output-monitor"]
    info("Rebuilding configuration...")
    nix_exec(rebuild_cmd, packages=rebuild_packages)
    success("Configuration rebuilt.")

def cleanup():
    banner()
    info("Cleaning up NixOS store... (deleting older than 3d)")
    nix_exec("sudo /usr/bin/env bash -p -c 'nix-collect-garbage --delete-older-than 3d --log-format internal-json -v |& nom --json'")
    success("NixOS store cleaned up.")

def update():
    banner()
    dotfiles = get_option("dotfiles")
    info("Updating NixOS...")
    nix_exec(f"cd {dotfiles}; sudo /usr/bin/env bash -p -c 'nix flake update --flake .# --log-format internal-json -v |& nom --json'")
    success("NixOS updated.")
